using RimWorld;
using System;

namespace Verse.AI
{
	public class JobGiver_ZombieFeed : ThinkNode_JobGiver
	{
		private const float WaitChance = 0.5f;

		private const int WaitTicks = 90;

		private const int MinMeleeChaseTicks = 420;

		private const int MaxMeleeChaseTicks = 900;

		protected override Job TryGiveJob(Pawn pawn)
		{
			bool flag = Rand.Value < 0.5f;
			Job result;
			if (flag)
			{
				result = new Job(JobDefOf.WaitCombat)
				{
					expiryInterval = 90
				};
			}
			else
			{
				bool downed = pawn.health.Downed;
				if (downed)
				{
					result = null;
				}
				else
				{
					Predicate<Thing> predicate = (Thing t) => t != pawn && t.Spawned && t is Building_Turret;
					ZombiePawn zombiePawn = pawn as ZombiePawn;
					float num = zombiePawn.notRaidingAttackRange;
					bool isRaiding = zombiePawn.isRaiding;
					if (isRaiding)
					{
						num = 999f;
					}
					Thing thing = GenClosest.ClosestThingReachable(pawn.Position, pawn.Map, ThingRequest.ForGroup(ThingRequestGroup.BuildingArtificial), PathEndMode.Touch, TraverseParms.For(zombiePawn, Danger.Deadly, TraverseMode.PassDoors, true), num, predicate, null, 5);
					bool flag2 = thing != null;
					if (flag2)
					{
						result = new Job(JobDefOf.AttackMelee, thing)
						{
							maxNumMeleeAttacks = 1,
							expiryInterval = Rand.Range(420, 900)
						};
					}
					else
					{
						Predicate<Thing> predicate2 = delegate(Thing t)
						{
							bool flag6 = t == pawn;
							bool result2;
							if (flag6)
							{
								result2 = false;
							}
							else
							{
								Pawn pawn2 = t as Pawn;
								result2 = (pawn2 != null && !pawn2.Downed && t.Spawned && !(pawn2 is ZombiePawn) && (pawn2.RaceProps.Humanlike || !pawn2.RaceProps.IsFlesh));
							}
							return result2;
						};
						Thing thing2 = GenClosest.ClosestThingReachable(pawn.Position, pawn.Map, ThingRequest.ForGroup(ThingRequestGroup.Pawn), PathEndMode.OnCell, TraverseParms.For(zombiePawn, Danger.Deadly, TraverseMode.PassDoors, false), num, predicate2, null, 50, -1, true);
						bool flag3 = thing2 != null;
						if (flag3)
						{
							Thing thing3;
                            using (PawnPath pawnPath = pawn.Map.pathFinder.FindPath(pawn.Position, new LocalTargetInfo(thing2), pawn, PathEndMode.OnCell))
                            {
                                thing3 = PawnPathUtility.FirstBlockingBuilding(pawnPath, out IntVec3 intVec, pawn);
                            }
							bool flag4 = thing3 != null;
							if (flag4)
							{
								result = new Job(JobDefOf.AttackMelee, thing3)
								{
									maxNumMeleeAttacks = 1,
									expiryInterval = Rand.Range(420, 900),
									locomotionUrgency = LocomotionUrgency.Walk
								};
								return result;
							}
							bool flag5 = thing2 != null;
							if (flag5)
							{
								result = new Job(JobDefOf.AttackMelee, thing2)
								{
									maxNumMeleeAttacks = 1,
									expiryInterval = Rand.Range(420, 900),
									locomotionUrgency = LocomotionUrgency.Walk
								};
								return result;
							}
						}
						result = null;
					}
				}
			}
			return result;
		}
	}
}
