using RimWorld;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Verse
{
    public class Alert_ColonistsBitten : Alert_Critical
	{
		protected override Color BGColor
		{
			get
			{
				float num = Pulser.PulseBrightness(0.5f, Pulser.PulseBrightness(0.5f, 0.6f));
				return new Color(num, num, num) * ZombieMod_Utility.zombieSkinColor;
			}
		}

        public override string GetExplanation()
        {
				StringBuilder stringBuilder = new StringBuilder();
				foreach (Pawn current in Find.Maps.SelectMany(m => m.mapPawns.FreeColonists))
				{
					BiteablePawn biteablePawn = current as BiteablePawn;
					bool flag = biteablePawn != null && biteablePawn.bitten;
					if (flag)
					{
						stringBuilder.AppendLine("    " + current.Name);
					}
				}
				return string.Format(Translator.Translate("ColonistsBittenList"), stringBuilder.ToString());
		}

        public override AlertReport GetReport()
        {
            List<Pawn> list = Find.Maps.SelectMany(m => m.mapPawns.FreeColonists).ToList<Pawn>();
            bool flag = list == null;
            AlertReport result;
            if (flag)
            {
                result = AlertReport.Inactive;
            }
            else
            {
                bool flag2 = list.Count == 0;
                if (flag2)
                {
                    result = AlertReport.Inactive;
                }
                else
                {
                    using (IEnumerator<Pawn> enumerator = Find.Maps.SelectMany(m => m.mapPawns.FreeColonists).GetEnumerator())
                    {
                        while (enumerator.MoveNext())
                        {
                            BiteablePawn biteablePawn = enumerator.Current as BiteablePawn;
                            bool flag3 = biteablePawn != null && biteablePawn.bitten;
                            if (flag3)
                            {
                                result = AlertReport.CulpritIs(biteablePawn);
                                return result;
                            }
                        }
                    }
                    result = AlertReport.Inactive;
                }
            }
            return result;
        }

        public Alert_ColonistsBitten() => this.defaultLabel = Translator.Translate("ColonistsBitten");
    }
}
