using RimWorld;
using System.Collections.Generic;

namespace Verse
{
    internal class BiteablePawn : Pawn
	{
		public bool bitten;

		private bool justSpawned = true;

		private bool runFirstTick;

		private bool firstSessionTick = true;

		private int currBittenDamgeTicks;

		public override void ExposeData()
		{
			base.ExposeData();
			Scribe_Values.Look<bool>(ref this.bitten, "bitten", false, false);
			Scribe_Values.Look<bool>(ref this.justSpawned, "justSpawned", false, false);
		}

		public override void PostApplyDamage(DamageInfo dinfo, float totalDamageDealt)
		{
			this.BiteChance(dinfo, 21f);
			bool flag = !this.Destroyed && this.Downed;
			if (flag)
			{
				this.BiteChance(dinfo, 50f);
			}
			base.PostApplyDamage(dinfo, 0f);
		}

		private void BiteChance(DamageInfo dinfo, float chance)
		{
			bool flag = !this.bitten && dinfo.Instigator is ZombiePawn;
			if (flag)
			{
                SkillRecord skill = this.skills.GetSkill(SkillDefOf.Melee);
                bool flag2 = UnityEngine.Random.Range(0f, 100f) < chance; // -  (float)skill.Level;
                if (flag2)
				{
					this.bitten = this.InfectPart(dinfo);
					bool flag3 = this.IsColonist && this.bitten;
					if (flag3)
					{
						Messages.Message(string.Format(Translator.Translate("ColonistInfected"), this.Name), MessageSound.SeriousAlert);
					}
				}
			}
		}

		public bool CanReanimate()
		{
			bool flag = this.health.hediffSet.GetBrain() == null;
			bool result;
			if (flag)
			{
				result = false;
			}
			else
			{
				List<BodyPartRecord> list = new List<BodyPartRecord>(this.health.hediffSet.GetInjuredParts());
				bool flag2 = list == null;
				if (flag2)
				{
					result = true;
				}
				else
				{
					using (List<BodyPartRecord>.Enumerator enumerator = list.GetEnumerator())
					{
						while (enumerator.MoveNext())
						{
							bool flag3 = enumerator.Current.def == this.health.hediffSet.GetBrain().def;
							if (flag3)
							{
								result = false;
								return result;
							}
						}
					}
					result = true;
				}
			}
			return result;
		}

		private bool InfectPart(DamageInfo dinfo)
		{
            Hediff hediff = new Hediff_Injury()
            {
                pawn = this
            };
            bool flag2 = false;
			int num = 0;
			BodyPartRecord bodyPartRecord = null;
            while (num < 5 && !flag2)
			{
				num++;
				bodyPartRecord = this.health.hediffSet.GetRandomNotMissingPart(dinfo.Def, dinfo.ForceHitPart?.height ?? BodyPartHeight.Middle, BodyPartDepth.Outside);
                bool flag3 = bodyPartRecord.def.IsSkinCovered(bodyPartRecord, this.health.hediffSet);
                if (flag3)
				{
					flag2 = true;
				}
			}
            bool flag4 = flag2;
			bool result;
            if (flag4)
			{
				this.health.AddHediff(ZombieMod_Utility.Z_Virus, bodyPartRecord, null);
				result = true;
			}
			else
			{
				result = false;
			}
            return result;
		}

		public override void Tick()
		{
			bool flag = this.firstSessionTick;
            if (flag)
			{
                bool flag2 = (!this.Map?.gameConditionManager?.ConditionIsActive(GameConditionDef.Named("ZombieVirusMapCondition")) ?? false) && (!this.Map?.gameConditionManager?.ConditionIsActive(GameConditionDef.Named("ZombieVirusAirborne")) ?? false);
                if (flag2)
				{
                    this.Map.gameConditionManager.RegisterCondition(GameConditionMaker.MakeCondition(GameConditionDef.Named("ZombieVirusMapCondition"), 999999, 0));
				}
                this.firstSessionTick = false;
			}
            bool flag3 = !this.runFirstTick;
			if (flag3)
			{
				bool flag4 = this.health.summaryHealth.SummaryHealthPercent <= 0f;
				if (flag4)
				{
					return;
				}
				this.runFirstTick = true;
				bool flag5 = this.justSpawned;
				if (flag5)
				{
					this.justSpawned = false;
				}
			}
            base.Tick();
            bool flag6 = this.health.summaryHealth.SummaryHealthPercent <= 0f;
			if (!flag6)
			{
				this.HandleBitten();
			}
		}

		private bool CheckStillBitten()
		{
			bool result;
			using (List<Hediff>.Enumerator enumerator = this.health.hediffSet.hediffs.GetEnumerator())
			{
				while (enumerator.MoveNext())
				{
					bool flag = enumerator.Current.def == ZombieMod_Utility.Z_Virus;
					if (flag)
					{
						result = true;
						return result;
					}
				}
			}
			result = false;
			return result;
		}

		private void HandleBitten()
		{
			bool flag = !this.bitten;
			if (!flag)
			{
				bool flag2 = this.health.summaryHealth.SummaryHealthPercent <= 0f;
				if (!flag2)
				{
					this.currBittenDamgeTicks++;
					bool flag3 = this.currBittenDamgeTicks >= 3000;
					if (flag3)
					{
						this.currBittenDamgeTicks = 0;
						bool isColonist = this.IsColonist;
						if (isColonist)
						{
							this.bitten = this.CheckStillBitten();
						}
					}
				}
			}
		}
	}
}
