using RimWorld;
using System.Collections.Generic;
using System.Linq;

namespace Verse
{
    public class GameCondition_Zombies : GameCondition
	{
		public static bool isAirborne;

		private int airborneTick = Find.TickManager.TicksGame;

		private int ticksSinceLastSpawn;

		private bool setCorpsesGreen;

		private int ticksUntilNextZombieRaid;

		private int minRaidTickBase = 180000;

		private int maxRaidTickBase = 500000;

		private int firstRaidTickMin = 75000;

        public override bool Expired => false;

        public GameCondition_Zombies()
		{
			this.duration = 999999999;
			GameCondition_Zombies.isAirborne = false;
			this.setCorpsesGreen = false;
			this.ticksUntilNextZombieRaid = this.GenerateTicksUntilNextRaid(true);
		}

		public void SetAirborne()
		{
			this.def = GameConditionDef.Named("ZombieVirusAirborne");
			GameCondition_Zombies.isAirborne = true;
			this.airborneTick = Find.TickManager.TicksGame;
		}

		public override void GameConditionTick()
		{
			base.GameConditionTick();
			this.setCorpsesGreen = ZombieMod_Utility.SetCorpsesGreen();
			this.HandleZombieRaid();
			this.HandleReanimation();
		}

		private void HandleZombieRaid()
		{
			this.ticksUntilNextZombieRaid--;
			bool flag = this.ticksUntilNextZombieRaid <= 0;
			if (flag)
			{
				this.ticksUntilNextZombieRaid = this.GenerateTicksUntilNextRaid();
                IncidentParms incidentParms = new IncidentParms()
                {
                    points = 0f
                };
                IncidentDef.Named("ZombieRaid").Worker.TryExecute(incidentParms);
			}
		}

		private float GetChallengeModifier()
		{
			float num = 2f - Find.Storyteller.difficulty.threatScale;
			bool flag = num < 0.7f;
			if (flag)
			{
				num = 0.7f;
			}
			return num;
		}

		private int GenerateTicksUntilNextRaid(bool first)
		{
			float challengeModifier = this.GetChallengeModifier();
			int num = UnityEngine.Random.Range((int)((float)this.minRaidTickBase * challengeModifier), (int)((float)this.maxRaidTickBase * challengeModifier));
			bool flag = first && num < this.firstRaidTickMin;
			if (flag)
			{
				num = this.firstRaidTickMin;
			}
			return num;
		}

        private int GenerateTicksUntilNextRaid() => this.GenerateTicksUntilNextRaid(false);

        private void HandleReanimation()
		{
			this.ticksSinceLastSpawn++;
			bool flag = this.ticksSinceLastSpawn > 25;
			if (flag)
			{
				this.ticksSinceLastSpawn = 0;
				bool flag2 = Find.Maps.SelectMany(m => m.listerThings.ThingsInGroup(ThingRequestGroup.Corpse)) != null;
				if (flag2)
				{
					List<Thing> list = Find.Maps.SelectMany(m => m.listerThings.ThingsInGroup(ThingRequestGroup.Corpse)).ToList();
					for (int i = list.Count - 1; i >= 0; i--)
					{
						Corpse corpse = list[i] as Corpse;
						bool flag3 = corpse.InnerPawn is BiteablePawn && ((BiteablePawn)corpse.InnerPawn).CanReanimate();
						if (flag3)
						{
							bool bitten = ((BiteablePawn)corpse.InnerPawn).bitten;
							if (bitten)
							{
								bool flag4 = corpse.Age > 1000 && UnityEngine.Random.value > 0.975f;
								if (flag4)
								{
									CompRottable comp = corpse.GetComp<CompRottable>();
									bool flag5 = comp == null || comp.Stage != RotStage.Dessicated;
									if (flag5)
									{
										ZombieMod_Utility.ZombieFactory(corpse);
									}
								}
							}
							else
							{
								bool flag6 = GameCondition_Zombies.isAirborne && Find.TickManager.TicksGame - this.airborneTick >= 24000 && corpse.Age > 12000 && UnityEngine.Random.value > 0.975f;
								if (flag6)
								{
									CompRottable comp2 = corpse.GetComp<CompRottable>();
									bool flag7 = comp2 == null || comp2.Stage != RotStage.Dessicated;
									if (flag7)
									{
										ZombieMod_Utility.ZombieFactory(corpse);
									}
								}
							}
						}
					}
				}
			}
		}

		public override void ExposeData()
		{
			base.ExposeData();
			Scribe_Values.Look<int>(ref this.airborneTick, "airborneTick", Find.TickManager.TicksGame, false);
			Scribe_Values.Look<bool>(ref GameCondition_Zombies.isAirborne, "isAirborne", false, false);
			Scribe_Values.Look<int>(ref this.ticksUntilNextZombieRaid, "ticksUntilNextZombieRaid", Find.TickManager.TicksGame, false);
		}
	}
}
