using RimWorld;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Verse.AI;
using Verse.AI.Group;

namespace Verse
{
    public class IncidentWorker_ZombieRaid : IncidentWorker_Raid
	{
		private enum HordeSize
		{
			Small = 175,
			Medium = 350,
			Large = 700,
			Massive = 1400,
			ALLTHEZOMBIES
		}

		private const int zombiePointValue = 35;

		protected override string GetLetterText(IncidentParms parms, List<Pawn> list)
		{
			IncidentWorker_ZombieRaid.HordeSize hordeSize = IncidentWorker_ZombieRaid.HordeSize.ALLTHEZOMBIES;
			bool flag = parms.points <= 175f;
			if (flag)
			{
				hordeSize = IncidentWorker_ZombieRaid.HordeSize.Small;
			}
			else
			{
				bool flag2 = parms.points <= 350f;
				if (flag2)
				{
					hordeSize = IncidentWorker_ZombieRaid.HordeSize.Medium;
				}
				else
				{
					bool flag3 = parms.points <= 700f;
					if (flag3)
					{
						hordeSize = IncidentWorker_ZombieRaid.HordeSize.Large;
					}
					else
					{
						bool flag4 = parms.points <= 1400f;
						if (flag4)
						{
							hordeSize = IncidentWorker_ZombieRaid.HordeSize.Massive;
						}
					}
				}
			}
			string text = "";
			IncidentWorker_ZombieRaid.HordeSize hordeSize2 = hordeSize;
			bool flag5 = hordeSize2 <= IncidentWorker_ZombieRaid.HordeSize.Medium;
			if (flag5)
			{
				bool flag6 = hordeSize2 != IncidentWorker_ZombieRaid.HordeSize.Small;
				if (flag6)
				{
					bool flag7 = hordeSize2 == IncidentWorker_ZombieRaid.HordeSize.Medium;
					if (flag7)
					{
						text += Translator.Translate("MediumHorde");
					}
				}
				else
				{
					text += Translator.Translate("SmallHorde");
				}
			}
			else
			{
				bool flag8 = hordeSize2 != IncidentWorker_ZombieRaid.HordeSize.Large;
				if (flag8)
				{
					bool flag9 = hordeSize2 != IncidentWorker_ZombieRaid.HordeSize.Massive;
					if (flag9)
					{
						bool flag10 = hordeSize2 == IncidentWorker_ZombieRaid.HordeSize.ALLTHEZOMBIES;
						if (flag10)
						{
							text += Translator.Translate("OverwhelmingHorde");
						}
					}
					else
					{
						text += Translator.Translate("MassiveHorde");
					}
				}
				else
				{
					text += Translator.Translate("LargeHorde");
				}
			}
			return text;
		}

        protected override string GetRelatedPawnsInfoLetterText(IncidentParms parms) => Translator.Translate("LetterRelatedPawnsRaidFriendly", new object[]
            {
                parms.faction.def.pawnsPlural
            });

        protected override bool CanFireNowSub(IIncidentTarget target) => ((target as Map)?.gameConditionManager.ConditionIsActive(GameConditionDef.Named("ZombieVirusMapCondition")) ?? false) || ((target as Map)?.gameConditionManager.ConditionIsActive(GameConditionDef.Named("ZombieVirusAirborne")) ?? false);

        protected override string GetLetterLabel(IncidentParms parms) => parms.raidStrategy.letterLabelEnemy;

        protected override LetterDef GetLetterDef() => LetterDefOf.BadUrgent;

        protected override void ResolveRaidStrategy(IncidentParms parms)
		{
			bool flag = parms.raidStrategy != null;
			if (!flag)
			{
				parms.raidStrategy = RaidStrategyDefOf.ImmediateAttack;
			}
		}

		protected override void ResolveRaidPoints(IncidentParms parms)
		{
			parms.points = StorytellerUtility.DefaultParmsNow(Find.Storyteller.def, IncidentCategory.ThreatBig, Find.AnyPlayerHomeMap).points;
			bool flag = parms.points > 0f;
			if (!flag)
			{
				parms.points = (float)Rand.Range(50, 300);
			}
		}

		protected override bool TryResolveRaidFaction(IncidentParms parms)
		{
			parms.faction = Find.FactionManager.FirstFactionOfDef(FactionDef.Named("Zombies"));
			return parms.faction != null;
		}

		public override bool TryExecute(IncidentParms parms)
		{
			this.ResolveRaidPoints(parms);
			bool flag = !this.TryResolveRaidFaction(parms);
			bool result;
			if (flag)
			{
				result = false;
			}
			else
			{
				this.ResolveRaidStrategy(parms);
				this.ResolveRaidArriveMode(parms);
				this.ResolveRaidSpawnCenter(parms);
                parms.target = Find.AnyPlayerHomeMap;
				IncidentParmsUtility.AdjustPointsForGroupArrivalParams(parms);
				List<Pawn> list = PawnGroupMakerUtility.GeneratePawns(PawnGroupKindDefOf.Normal, IncidentParmsUtility.GetDefaultPawnGroupMakerParms(parms), true).ToList<Pawn>();
				bool flag2 = list.Count == 0;
				if (flag2)
				{
					Log.Error("Got no pawns spawning raid from parms " + parms);
					result = false;
				}
				else
				{
					TargetInfo targetInfo = TargetInfo.Invalid;
					bool flag3 = parms.raidArrivalMode == PawnsArriveMode.CenterDrop || parms.raidArrivalMode == PawnsArriveMode.EdgeDrop;
					if (flag3)
					{
						DropPodUtility.DropThingsNear(parms.spawnCenter, parms.target as Map, list.Cast<Thing>(), parms.raidPodOpenDelay, false, true, true);
						targetInfo = new TargetInfo(parms.spawnCenter, parms.target as Map);
					}
					else
					{
						foreach (Pawn current in list)
						{
							IntVec3 intVec = CellFinder.RandomClosewalkCellNear(parms.spawnCenter, parms.target as Map, 8);
							GenSpawn.Spawn(current, intVec, parms.target as Map);
							targetInfo = current;
						}
					}
					StringBuilder stringBuilder = new StringBuilder();
					stringBuilder.AppendLine("Points = " + parms.points.ToString("F0"));
					foreach (Pawn current2 in list)
					{
						string str = (current2.equipment == null || current2.equipment.Primary == null) ? "unarmed" : current2.equipment.Primary.LabelCap;
						stringBuilder.AppendLine(current2.KindLabel + " - " + str);
					}
					Find.LetterStack.ReceiveLetter(this.GetLetterLabel(parms), this.GetLetterText(parms, list), this.GetLetterDef(), targetInfo, stringBuilder.ToString());
					bool flag4 = this.GetLetterDef() == LetterDefOf.BadUrgent;
					if (flag4)
					{
						TaleRecorder.RecordTale(TaleDefOf.RaidArrived, new object[0]);
					}
                    string info = this.GetRelatedPawnsInfoLetterText(parms);
                    PawnRelationUtility.Notify_PawnsSeenByPlayer(list, out info, true);
					Lord lord = LordMaker.MakeNewLord(parms.faction, parms.raidStrategy.Worker.MakeLordJob(parms, parms.target as Map), parms.target as Map, list);
					AvoidGridMaker.RegenerateAvoidGridsFor(parms.faction, parms.target as Map);
					LessonAutoActivator.TeachOpportunity(ConceptDefOf.EquippingWeapons, OpportunityType.Important);
					bool flag5 = DebugViewSettings.drawStealDebug && FactionUtility.HostileTo(parms.faction, Faction.OfPlayer);
					if (flag5)
					{
						Log.Message(string.Concat(new object[]
						{
							"Market value threshold to start stealing: ",
							StealAIUtility.StartStealingMarketValueThreshold(lord),
							" (colony wealth = ",
							(parms.target as Map).wealthWatcher.WealthTotal,
							")"
						}));
					}
					result = true;
				}
			}
			return result;
		}

        protected override void ResolveRaidArriveMode(IncidentParms parms) => parms.raidArrivalMode = PawnsArriveMode.EdgeWalkIn;
    }
}
