using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using UnityEngine;
using Verse.AI;

namespace Verse
{
	[StaticConstructorOnStartup]
	public static class ZombieMod_Utility
	{
		public const float biteChance = 0.1f;

		public const int bittenDamage = 1;

		public const int bittenDamageTicks = 3000;

		public static Color zombieSkinColor = new Color(0.37f, 0.48f, 0.35f, 1f);

		public static bool addedPawnComps = false;

		public static HediffDef Z_Virus = DefDatabase<HediffDef>.GetNamed("ZombieInfection", true);

		public static void ZombieFactory(Corpse corpse)
		{
			Thing thing = (ZombiePawn)ZombieMod_Utility.GenerateZombiePawnFromSource(corpse.InnerPawn);
			IntVec3 position = corpse.Position;
			Building building = StoreUtility.StoringBuilding(corpse);
			bool flag = building != null;
			if (flag)
			{
				((Building_Storage)building).Notify_LostThing(corpse);
			}
			GenSpawn.Spawn(thing, position, corpse.Map);
			corpse.Destroy(0);
		}

		public static Pawn GenerateZombiePawnFromSource(Pawn sourcePawn)
		{
			PawnKindDef pawnKindDef = PawnKindDef.Named("Zombie");
			Faction factionDirect = Find.FactionManager.FirstFactionOfDef(DefDatabase<FactionDef>.GetNamed("Zombies", true));
			Pawn pawn = (Pawn)ThingMaker.MakeThing(pawnKindDef.race, null);
			pawn.kindDef = pawnKindDef;
			pawn.SetFactionDirect(factionDirect);

            pawn.InitializeComps();

			PawnComponentsUtility.CreateInitialComponents(pawn);
            pawn.InitializeComps();

            bool toolUser = pawn.RaceProps.ToolUser;
			if (toolUser)
			{
				pawn.equipment = new Pawn_EquipmentTracker(pawn);
				pawn.carryTracker = new Pawn_CarryTracker(pawn);
				pawn.apparel = new Pawn_ApparelTracker(pawn);
				pawn.inventory = new Pawn_InventoryTracker(pawn);
			}
			bool humanlike = pawn.RaceProps.Humanlike;
			if (humanlike)
			{
				pawn.ownership = new Pawn_Ownership(pawn);
				pawn.skills = new Pawn_SkillTracker(pawn);
				pawn.relations = new Pawn_RelationsTracker(pawn);
				pawn.story = new Pawn_StoryTracker(pawn);
				pawn.workSettings = new Pawn_WorkSettings(pawn);
			}
			bool flag = pawn.RaceProps.intelligence <= 0;
			if (flag)
			{
				pawn.caller = new Pawn_CallTracker(pawn);
			}

			pawn.gender = sourcePawn.gender;
			ZombieMod_Utility.GenerateRandomAge(pawn);
			pawn.needs.SetInitialLevels();
			bool humanlike2 = pawn.RaceProps.Humanlike;
            if (humanlike2)
            {
                string headGraphicPath = sourcePawn.story.HeadGraphicPath;
                typeof(Pawn_StoryTracker).GetField("headGraphicPath", BindingFlags.Instance | BindingFlags.NonPublic).SetValue(pawn.story, headGraphicPath);
                pawn.story.melanin = sourcePawn.story.melanin;
                pawn.story.crownType = sourcePawn.story.crownType;
                pawn.story.hairColor = sourcePawn.story.hairColor;
                pawn.story.bodyType = sourcePawn.story.bodyType;
                ZombieMod_Utility.GenerateZombieBioFromSource(pawn, sourcePawn);
                pawn.story.hairDef = sourcePawn.story.hairDef;
                foreach (Trait current in sourcePawn.story.traits.allTraits)
                {
                    pawn.story.traits.GainTrait(current);
                }
                typeof(PawnGenerator).GetMethod("GenerateSkills", BindingFlags.NonPublic | BindingFlags.Static).Invoke(null, new object[] { pawn });
			}
			ZombieMod_Utility.GenerateZombieApparelFromSource(pawn, sourcePawn);
			PawnInventoryGenerator.GenerateInventoryFor(pawn, new PawnGenerationRequest(pawn.kindDef, pawn.Faction, PawnGenerationContext.NonPlayer, sourcePawn.Tile, false, false, false, false, false, true, 1f, false, true, false, false, false, null, null, null, null, null, null));
			((ZombiePawn)pawn).isRaiding = false;
			bool isColonist = sourcePawn.IsColonist;
			if (isColonist)
			{
				((ZombiePawn)pawn).wasColonist = true;
			}
			ZombieMod_Utility.Zombify(pawn);
			ZombieMod_Utility.SetZombieName(pawn);
			return pawn;
		}

		private static void GenerateRandomAge(Pawn pawn)
		{
			int num = 0;
			int num2;
			while (true)
			{
				bool flag = pawn.RaceProps.ageGenerationCurve != null;
				if (flag)
				{
					num2 = Mathf.RoundToInt(Rand.ByCurve(pawn.RaceProps.ageGenerationCurve, 200));
				}
				else
				{
					bool isMechanoid = pawn.RaceProps.IsMechanoid;
					if (isMechanoid)
					{
						num2 = Rand.Range(0, 2500);
					}
					else
					{
						bool flag2 = !pawn.RaceProps.Animal;
						if (flag2)
						{
							break;
						}
						num2 = Rand.Range(1, 10);
					}
				}
				num++;
				bool flag3 = num > 100;
				if (flag3)
				{
					goto Block_4;
				}
				if (num2 <= pawn.kindDef.maxGenerationAge && num2 >= pawn.kindDef.minGenerationAge)
				{
					goto Block_6;
				}
			}
			Log.Warning("Didn't get age for " + pawn);
			return;
			Block_4:
			Log.Error("Tried 100 times to generate age for " + pawn);
			Block_6:
			pawn.ageTracker.AgeBiologicalTicks = ((long)((float)num2 * 3600000f) + (long)Rand.Range(0, 3600000));
			bool flag4 = Rand.Value < pawn.kindDef.backstoryCryptosleepCommonality;
			int num3;
			if (flag4)
			{
				float value = Rand.Value;
				bool flag5 = value < 0.7f;
				if (flag5)
				{
					num3 = Rand.Range(0, 100);
				}
				else
				{
					bool flag6 = value < 0.95f;
					if (flag6)
					{
						num3 = Rand.Range(100, 1000);
					}
					else
					{
						int num4 = GenDate.YearsPassed + GenDate.DefaultStartingYear - 2026 - pawn.ageTracker.AgeBiologicalYears;
						num3 = Rand.Range(1000, num4);
					}
				}
			}
			else
			{
				num3 = 0;
			}
			long num5 = (long)GenTicks.TicksAbs - pawn.ageTracker.AgeBiologicalTicks;
			num5 -= (long)num3 * 3600000L;
			pawn.ageTracker.BirthAbsTicks = num5;
			bool flag7 = pawn.ageTracker.AgeBiologicalTicks > pawn.ageTracker.AgeChronologicalTicks;
			if (flag7)
			{
				pawn.ageTracker.AgeChronologicalTicks = pawn.ageTracker.AgeBiologicalTicks;
			}
		}

		public static void SetZombieName(Pawn pawn)
		{
			NameTriple nameTriple = pawn.Name as NameTriple;
			bool flag = nameTriple.Nick.Contains(Translator.Translate("Zombie"));
			if (!flag)
			{
				bool wasColonist = ((ZombiePawn)pawn).wasColonist;
				if (wasColonist)
				{
					pawn.Name =new NameTriple(nameTriple.First, string.Concat(new string[]
					{
						"* ",
						Translator.Translate("Zombie"),
						" ",
						nameTriple.Nick,
						" *"
					}), nameTriple.Last);
				}
				else
				{
					bool flag2 = !((ZombiePawn)pawn).isRaiding && !((ZombiePawn)pawn).wasRaiding;
					if (flag2)
					{
						pawn.Name =new NameTriple(nameTriple.First, Translator.Translate("Zombie") + " " + nameTriple.Nick, nameTriple.Last);
					}
					else
					{
						pawn.Name =new NameTriple(nameTriple.First, Translator.Translate("Zombie"), nameTriple.Last);
					}
				}
			}
		}

		private static NameTriple CurPawnName(Pawn pawn)
		{
			NameTriple nameTriple = pawn.Name as NameTriple;
			bool flag = nameTriple != null;
			if (flag)
			{
				return new NameTriple(nameTriple.First, nameTriple.Nick, nameTriple.Last);
			}
			throw new InvalidOperationException();
		}

		public static bool Zombify(Pawn pawn)
		{
			bool flag = pawn.Drawer == null;
			bool result;
			if (flag)
			{
				result = false;
			}
			else
			{
				bool flag2 = pawn.Drawer.renderer == null;
				if (flag2)
				{
					result = false;
				}
				else
				{
					bool flag3 = pawn.Drawer.renderer.graphics == null;
					if (flag3)
					{
						result = false;
					}
					else
					{
						bool flag4 = !pawn.Drawer.renderer.graphics.AllResolved;
						if (flag4)
						{
							pawn.Drawer.renderer.graphics.ResolveAllGraphics();
						}
						bool flag5 = pawn.Drawer.renderer.graphics.headGraphic == null;
						if (flag5)
						{
							result = false;
						}
						else
						{
							bool flag6 = pawn.Drawer.renderer.graphics.nakedGraphic == null;
							if (flag6)
							{
								result = false;
							}
							else
							{
								bool flag7 = pawn.Drawer.renderer.graphics.headGraphic.path == null;
								if (flag7)
								{
									result = false;
								}
								else
								{
									bool flag8 = pawn.Drawer.renderer.graphics.nakedGraphic.path == null;
									if (flag8)
									{
										result = false;
									}
									else
									{
										Graphic nakedBodyGraphic = GraphicGetter_NakedHumanlike.GetNakedBodyGraphic(pawn.story.bodyType, ShaderDatabase.CutoutSkin, ZombieMod_Utility.zombieSkinColor);
										Graphic headGraphic = GraphicDatabase.Get<Graphic_Multi>(pawn.story.HeadGraphicPath, ShaderDatabase.CutoutSkin, Vector2.one, ZombieMod_Utility.zombieSkinColor);
										pawn.Drawer.renderer.graphics.headGraphic = headGraphic;
										pawn.Drawer.renderer.graphics.nakedGraphic = nakedBodyGraphic;
										bool flag9 = ((ZombiePawn)pawn).isRaiding && UnityEngine.Random.Range(0f, 100f) < 33f;
										if (flag9)
										{
											((ZombiePawn)pawn).isRaiding = false;
											((ZombiePawn)pawn).wasRaiding = true;
										}
										result = true;
									}
								}
							}
						}
					}
				}
			}
			return result;
		}

		public static void GenerateZombieBioFromSource(Pawn zombie, Pawn sourcePawn)
		{
			sourcePawn.Name = ZombieMod_Utility.CurPawnName(sourcePawn);
			NameTriple name = sourcePawn.Name as NameTriple;
			zombie.Name = name;
			zombie.story.childhood = sourcePawn.story.childhood;
			zombie.story.adulthood = sourcePawn.story.adulthood;
		}

		public static bool SetCorpsesGreen()
		{
			bool flag = Find.Maps.SelectMany(m => m.listerThings.ThingsInGroup(ThingRequestGroup.Corpse)) != null;
			bool result;
			if (flag)
			{
                using (List<Thing>.Enumerator enumerator = Find.Maps.SelectMany(m => m.listerThings.ThingsInGroup(ThingRequestGroup.Corpse)).ToList().GetEnumerator())
                {
                    while (enumerator.MoveNext())
                    {
                        Corpse corpse = enumerator.Current as Corpse;
                        bool flag2 = corpse != null && corpse.InnerPawn != null && corpse.Spawned && corpse.InnerPawn is ZombiePawn && !ZombieMod_Utility.Zombify(corpse.InnerPawn as ZombiePawn);
                        if (flag2)
                        {
                            result = false;
                            return result;
                        }
                    }
                }
				result = true;
			}
			else
			{
				result = true;
			}
			return result;
		}

		public static void GenerateZombieApparelFromSource(Pawn zombie, Pawn sourcePawn)
		{
			bool flag = sourcePawn.apparel == null || sourcePawn.apparel.WornApparelCount == 0;
			if (!flag)
			{
				foreach (Apparel current in sourcePawn.apparel.WornApparel)
				{
					bool madeFromStuff = current.def.MadeFromStuff;
					Apparel apparel;
					if (madeFromStuff)
					{
						apparel = (Apparel)ThingMaker.MakeThing(current.def, current.Stuff);
					}
					else
					{
						apparel = (Apparel)ThingMaker.MakeThing(current.def, null);
					}
					apparel.DrawColor = new Color(current.DrawColor.r, current.DrawColor.g, current.DrawColor.b, current.DrawColor.a);
					zombie.apparel.Wear(apparel, true);
				}
			}
		}

		public static void GenerateZombieInjuriesFromSource(Pawn zombie, Pawn sourcePawn)
		{
		}
	}
}
