using RimWorld;
using System;
using Verse.AI;

namespace Verse
{
    internal class ZombiePawn : Pawn
	{
		private bool setZombie;

		public bool isRaiding = true;

		public bool wasRaiding;

		public bool wasColonist;

		public float notRaidingAttackRange = 15f;

        public ZombiePawn() => this.Init();

        private void Init()
		{
			this.setZombie = false;
			this.pather = new Pawn_PathFollower(this);
			this.stances = new Pawn_StanceTracker(this);
			this.health = new Pawn_HealthTracker(this);
			this.jobs = new Pawn_JobTracker(this);
			this.filth = new Pawn_FilthTracker(this);
			this.notRaidingAttackRange = UnityEngine.Random.Range(15f, 250f);
		}

		public override void ExposeData()
		{
			base.ExposeData();
			Scribe_Values.Look<bool>(ref this.isRaiding, "isRaiding", true, false);
			Scribe_Values.Look<bool>(ref this.wasColonist, "wasColonist", false, false);
			Scribe_Values.Look<float>(ref this.notRaidingAttackRange, "notRaidingAttackRange", 100f, false);
		}

		public override void PreApplyDamage(DamageInfo dinfo, out bool absorbed)
		{
			this.health.PreApplyDamage(dinfo, out absorbed);
			bool flag = !this.Destroyed && (dinfo.Def == DamageDefOf.Cut || dinfo.Def == DamageDefOf.Stab);
			if (flag)
			{
				float num = 0f;
				float num2 = 0f;
				bool flag2 = dinfo.Instigator != null && dinfo.Instigator is Pawn && ((Pawn)dinfo.Instigator).IsColonist;
				if (flag2)
				{
					Pawn pawn = dinfo.Instigator as Pawn;
					bool flag3 = pawn.skills != null;
					if (flag3)
					{
						SkillRecord skill = pawn.skills.GetSkill(SkillDefOf.Melee);
						num = (float)(skill.Level * 2);
						num2 = (float)skill.Level / 20f * 3f;
					}
					bool flag4 = UnityEngine.Random.Range(0f, 100f) < 20f + num;
					if (flag4)
					{
						dinfo.SetAmount(999);
						dinfo.SetForcedHitPart(this.health.hediffSet.GetBrain());
						dinfo.Def.Worker.Apply(dinfo, this);
					}
					else
					{
						dinfo.SetAmount((int)((float)dinfo.Amount * (1f + num2)));
					}
				}
			}
		}

		public override void Tick()
		{
			try
			{
				bool flag = DebugSettings.noAnimals && this.RaceProps.Animal;
				if (flag)
				{
					this.Destroy(0);
				}
				else
				{
					bool flag2 = !this.Downed;
					if (flag2)
					{
						bool flag3 = Find.TickManager.TicksGame % 250 == 0;
						if (flag3)
						{
							this.TickRare();
						}
						bool spawned = this.Spawned;
						if (spawned)
						{
							this.pather.PatherTick();
						}
                        this.Drawer.DrawTrackerTick();
						this.health.HealthTick();
						this.records.RecordsTick();
						bool spawned2 = this.Spawned;
						if (spawned2)
						{
							this.stances.StanceTrackerTick();
						}
						bool spawned3 = this.Spawned;
						if (spawned3)
						{
							this.verbTracker.VerbsTick();
						}
						bool spawned4 = this.Spawned;
						if (spawned4)
						{
							this.natives.NativeVerbsTick();
						}
						bool flag4 = this.equipment != null;
						if (flag4)
						{
							this.equipment.EquipmentTrackerTick();
						}
						bool flag5 = this.apparel != null;
						if (flag5)
						{
							this.apparel.ApparelTrackerTick();
						}
                        bool spawned5 = this.Spawned;
						if (spawned5)
						{
							this.jobs.JobTrackerTick();
						}
						bool flag6 = !this.Dead;
						if (flag6)
						{
							this.carryTracker.CarryHandsTick();
						}
						bool flag7 = this.skills != null;
						if (flag7)
						{
							this.skills.SkillsTick();
						}
						bool flag8 = this.inventory != null;
						if (flag8)
						{
							this.inventory.InventoryTrackerTick();
						}
					}
				}
				bool flag9 = this.needs != null && this.needs.food != null && this.needs.food.CurLevel <= 0.95f;
				if (flag9)
				{
					this.needs.food.CurLevel = 1f;
				}
				bool flag10 = this.needs != null && this.needs.joy != null && this.needs.joy.CurLevel <= 0.95f;
				if (flag10)
				{
					this.needs.joy.CurLevel = 1f;
				}
				bool flag11 = this.needs != null && this.needs.beauty != null && this.needs.beauty.CurLevel <= 0.95f;
				if (flag11)
				{
					this.needs.beauty.CurLevel = 1f;
				}
				bool flag12 = this.needs != null && this.needs.comfort != null && this.needs.comfort.CurLevel <= 0.95f;
				if (flag12)
				{
					this.needs.comfort.CurLevel = 1f;
				}
				bool flag13 = this.needs != null && this.needs.rest != null && this.needs.rest.CurLevel <= 0.95f;
				if (flag13)
				{
					this.needs.rest.CurLevel = 1f;
				}
				bool flag14 = this.needs != null && this.needs.mood != null && this.needs.mood.CurLevel <= 0.45f;
				if (flag14)
				{
					this.needs.mood.CurLevel = 0.5f;
				}
				bool flag15 = !this.setZombie;
				if (flag15)
				{
					this.mindState.mentalStateHandler.neverFleeIndividual = true;
					this.setZombie = ZombieMod_Utility.Zombify(this);
					ZombieMod_Utility.SetZombieName(this);
				}
				bool flag16 = this.Downed || this.health.Downed || this.health.InPainShock;
				if (flag16)
				{
					DamageInfo damageInfo = new DamageInfo(DamageDefOf.Blunt, 9999, -1,this, null, null);
					damageInfo.SetForcedHitPart(this.health.hediffSet.GetBrain());
					base.TakeDamage(damageInfo);
				}
			}
			catch (Exception)
			{
			}
		}
	}
}
